
from typing import Optional

import requests
from uuid import UUID

from vulavula.client.nlu import NLU
from vulavula.client.sentiment import Sentiment
from vulavula.client.transcribe import Transcribe
from vulavula.client.transport import Transport
from vulavula.client.vulavula_search import VulavulaSearch
from vulavula.client.translation import Translation
from vulavula.config import settings
from vulavula.models.types import (EntityRecognitionRequestData, SentimentAnalysisRequestData, \
    IntentClassificationRequestData , CreateKnowledgeBaseRequest,TranslationRequest)


class VulavulaClient:
    """
    A client for interacting with the Vulavula API.

    This client provides access to a variety of services such as natural language processing,
    sentiment analysis, transcription, and file transport. It simplifies the interaction with different
    components of the Vulavula API by encapsulating them into easy-to-use sub-client classes.

    Parameters:
        client_token (str): The API token used for authenticating with the API.
        base_url (Optional[str]): The base URL for the API endpoints. If not provided,
                                  defaults to the URL specified in the project's settings (add in .env).

    Attributes:
        base_url (str): The API base URL used for all requests.
        session (requests.Session): The session used for HTTP requests to optimize connection reuse.
        headers (dict): Default headers to include in every API request, including the authentication token.
        nlu (NLU): Sub-client for Natural Language Understanding features.
        sentiments (Sentiment): Sub-client for sentiment analysis features.
        transcribe_client (Transcribe): Sub-client for audio transcription services.
        transport (Transport): Sub-client for file transport services.

    Usage:
         client = VulavulaClient("<INSERT_TOKEN>")
         response = client.sentiments.get_sentiments({"encoded_text": "I hate the rain. But I love the sun."})
         print(response)
    """
    def __init__(self, client_token, base_url=None):
        """
        Initializes a new instance of the VulavulaClient.

        The client can optionally accept a custom base URL.

        Args:
            client_token (str): The token required to authenticate requests to the API.
            base_url (str, optional): A custom URL to use instead of the default API URL.
        """
        self.base_url = base_url if base_url else settings.BASE_URL
        self.session = requests.Session()
        self.headers = {
            "X-CLIENT-TOKEN": client_token,
            "Content-Type": "application/json"
        }
        self.session.headers.update(self.headers)
        self.nlu = NLU(client_token, self.base_url, session=self.session)
        self.sentiments = Sentiment(client_token, self.base_url, session=self.session)
        self.transcribe_client = Transcribe(client_token, self.base_url, session=self.session)
        self.transport = Transport(client_token, self.base_url, session=self.session)
        self.search = VulavulaSearch(client_token, self.base_url, session=self.session)
        self.translation = Translation(client_token, self.base_url, session=self.session)

    def upload_file(self, file_path: str):
        return self.transport.upload_file(file_path)

    def transcribe_process(self, upload_id: UUID, webhook: Optional[str] = None, language_code: Optional[str] = None):
        return self.transcribe_client.transcribe_process(upload_id, webhook=webhook, language_code=language_code)

    def transcribe(self, file_path: str, *, webhook: Optional[str] = None, language_code: Optional[str] = None):
        return self.transcribe_client.transcribe(file_path, webhook=webhook, language_code=language_code)

    def get_transcribed_text(
            self,
            upload_id: UUID
    ):
        return self.transcribe_client.get_transcribed_text(upload_id)

    def get_sentiments(self, data: SentimentAnalysisRequestData):
        return self.sentiments.get_sentiments(data)

    def get_entities(self, data: EntityRecognitionRequestData):
        return self.nlu.get_entities(data)

    def classify(self, data: IntentClassificationRequestData):
        return self.nlu.classify(data)

    def get_intents(self, data: IntentClassificationRequestData):
        return self.nlu.get_intents(data)

    def create_knowledgebase(self, data: CreateKnowledgeBaseRequest):
        return self.search.create_collection(data)

    def create_documents(self,file_path: str, language: str=None):
        return self.search.upload_and_extract_text(file_path, language)

    def query(self,query: str, language: str=None):
        return self.search.search_query(query, language)

    def translate(self, data:TranslationRequest):
        return self.translation.translate(data)



from pydantic import (
    BaseSettings
)


class Settings(BaseSettings):
    """
    use this http://127.0.0.1:8000/api/v1 for local testing
    """
    BASE_URL: str = "https://vulavula-services.lelapa.ai/api/v1"

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"
        case_sensitive = True
        extra = 'ignore'


settings = Settings()



from typing import Union, Dict

import requests

from vulavula.common.response_handler import APIResponseHandler
from vulavula.config import settings
from vulavula.models.types import TranslationRequest


class Translation:
    """
    Handles translation for the Vulavula API, providing functionalities to translate
    text from one language to another.

    The Translation class allows for the translation of given text, facilitating
    the interpretation of text in different languages.

    Parameters:
        client_token (str): The API token required for authenticating all requests.
        base_url (str): The base URL for the API endpoints.
        session (requests.Session): The session used for HTTP requests to optimize connection reuse.

    Example:
        translation_client = Translation("your_api_token", "https://beta-vulavula-services.lelapa.ai/api/v1/")
        translation_results = translation_client.translate({
            "input_text": "this is a test",
            "source_lang": "en",
            "target_lang": "sw"
        })
        print(translation_results)
    """
    def __init__(self, client_token,  base_url=None, session=None):
        self.base_url = base_url if base_url else settings.BASE_URL
        self.headers = {
            "X-CLIENT-TOKEN": client_token,
            "Content-Type": "application/json"
        }
        self._handle_response = APIResponseHandler.handle_response
        self.session = session if session else requests.Session()
        self.session.headers.update(self.headers)

    def translate(self, data: Union[Dict[str, str], TranslationRequest]):
        """
        Sends a request to the API to perform translation on the provided text.

        Parameters:
            data (TranslationRequest): An instance of TranslationRequest containing the text to be translated,
                                       the source language, the target language, and an optional webhook URL.

        Returns:
            dict: The response from the server after processing the translation request.

        Example:
            data = {
                "input_text": "this is a test",
                "source_lang": "en",
                "target_lang": "sw"
            }
            try:
                translation_result = client.translate(data)
                print("Translation Result:", translation_result)
            except Exception as e:
                print(f"Error during translation: {e}")
        """
        if isinstance(data, TranslationRequest):
            data = data.__dict__
        url = f"{self.base_url}/translate/process"
        response = self.session.post(url, headers=self.headers, json=data)
        return self._handle_response(response)



from typing import Union, Dict
from vulavula.common.response_handler import APIResponseHandler
from vulavula.config import settings
from vulavula.models.types import CreateKnowledgeBaseRequest
import requests
import os
import logging
class VulavulaSearch:

    def __init__(self, client_token,  base_url=None, session=None):
        self.base_url = base_url if base_url else settings.BASE_URL
        self.headers = {
            "X-CLIENT-TOKEN": client_token,
            "accept": "application/json"
        }
        self.client_token= client_token
        self._handle_response = APIResponseHandler.handle_response
        self.session = session if session else requests.Session()
        self.session.headers.update(self.headers)
        self.collection = None
        self.logger = logging.getLogger("vulavulaSearch")

    def create_collection(self, data: Union[str, CreateKnowledgeBaseRequest]):
        """
        Sends a request to the API to create a knowledge base with the provided data.

        Parameters:
            data (Union[str, CreateKnowledgeBaseRequest]): An instance of CreateKnowledgeBaseRequest containing the data to create a knowledge base.
                                             This ensures that the 'collection' keys exist.

        Returns:
            dict: The response from the server after processing the knowledge base creation request.

        Example:
            data = CreateKnowledgeBaseRequest(collection="myCollection")
            try:
                kb_result = client.create_knowledge_base(data)
                print("Knowledge Base Creation Result:", kb_result)
            except Exception as e:
                print(f"Error during knowledge base creation: {e}")
        """
        if isinstance(data, CreateKnowledgeBaseRequest):
            data = data.__dict__
        url = f"{self.base_url}/search/create-knowledgebase"
        response = self.session.post(url, headers=self.headers, json=data)
        self.collection = data['collection']
        return self._handle_response(response)

    def upload_and_extract_text(self, file_path: str, language: str = None) -> dict:
        # Upload the file
        upload_url = f"{self.base_url}/search/upload-file"

        if not os.path.isfile(file_path):
            self.logger.info(f"File does not exist: {file_path}")
            return {"error": "File not found"}

        try:
            with open(file_path, "rb") as file:
                files = {"file": (file_path, file, "application/pdf")}
                upload_response = requests.post(upload_url, headers=self.headers, files=files)

            if upload_response.status_code != 200:
                self.logger.info(f"Failed to upload file. Status code: {upload_response.status_code}")
                return self._handle_response(upload_response)

            # Assuming the response contains the extracted documents
            documents = upload_response.json().get("documents", [])

            # Create documents with the extracted text
            create_documents_url = f"{self.base_url}/search/create-documents"
            data = {
                "webhook": "string",
                "documents": documents,
                "language": language,
                "collection": self.collection
            }

            create_response = requests.post(create_documents_url, headers=self.headers, json=data)

            if create_response.status_code == 200:
                self.logger.info("Documents created successfully.")
            else:
                self.logger.info(f"Failed to create documents. Status code: {create_response.status_code}")

            return self._handle_response(create_response)

        except Exception as e:
            self.logger.info(f"An error occurred: {e}")
            return {"error": str(e)}

    def search_query(self, query: str, language: str = None) -> dict:
        url = f"{self.base_url}/search/search"
        data = {
            "query": query,
            "webhook": "string_url",
            "language": language,
            "collection": self.collection
        }
        headers = {
            "accept": "application/json",
            "X-CLIENT-TOKEN": self.client_token
        }
        response = requests.post(url, headers=headers, json=data)
        return self._handle_response(response)


from .entity_recognition import EntityRecognitionRequestData
from .sentiment_analysis import SentimentAnalysisRequestData
from .intent_classification import IntentClassificationRequestData, IntentClassificationResponse
from .vulavula_search import  (CreateDocumentsRequest,CreateKnowledgeBaseRequest,
                               SearchQueryRequest,QueryKnowledgeBase,UploadPDFRequest)
from .translation import TranslationRequest




from dataclasses import dataclass
from typing import Dict, List, Optional
@dataclass
class CreateKnowledgeBaseRequest:
    collection: str
@dataclass
class CreateDocumentsRequest:
    documents: List[str]
    collection: Optional[str]
    language: str
@dataclass
class SearchQueryRequest:
    query: str
    language: Optional[str]
    collection: str

@dataclass
class QueryKnowledgeBase:
    query: str

@dataclass
class UploadPDFRequest:
    file: bytes
    filename: str